﻿using UnityEngine;
using System.Collections;

public class MouseManager : MonoBehaviour {

	// Use this for initialization
	void Start () {
	
	}

    public GameObject thePrefabToSpawn;

    public LayerMask snapPointLayerMask;

    public ComponentKeybindDialog ComponentKeybindDialog;
	
	// Update is called once per frame
	void Update () {
	
        // Was the mouse pressed down this frame?

        if(Input.GetMouseButtonDown(0))
        {
            CheckLeftClick();
        }

        if(Input.GetMouseButtonDown(1))
        {
            CheckRightClick();
        }
	}

    Collider DoRaycast()
    {
        Camera theCamera = Camera.main;

        Ray ray = theCamera.ScreenPointToRay( Input.mousePosition );

        RaycastHit hitInfo;

        if( Physics.Raycast( ray, out hitInfo ) )
        {
            return hitInfo.collider;
        }

        return null;
    }

    void CheckRightClick()
    {
        Collider theCollider = DoRaycast();

        if(theCollider == null)
        {
            return;
        }

        // We have right-clicked on something.  Is it a KeybindableComponent?
        // We need to check the PARENT of the object with the collider, given
        // how we have assembled our prefabs.

        GameObject shipPart = FindShipPart(theCollider);

        if(shipPart == null)
        {
            // We clicked on something that doesn't have a parent, so it's probably
            // not a valid part of our ship
            return;
        }


        KeybindableComponent kc = shipPart.GetComponent<KeybindableComponent>();

        if(kc == null)
        {
            // This object isn't keybindable.
            return;
        }

        // If we get to this point, we have right-clicked on something keybindable

        ComponentKeybindDialog.OpenDialog( kc );

    }

    GameObject FindShipPart(Collider collider)
    {
        Transform curr = collider.transform;

        while(curr != null)
        {
            if(curr.gameObject.tag == "ShipPart")
            {
                return curr.gameObject;
            }

            curr = curr.parent;
        }

        return null;
    }

    void CheckLeftClick()
    {
        // Yes, the left mouse button was pressed this frame
        // Is the mouse over a cube

        Collider theCollider = DoRaycast();

        if(theCollider == null)
        {
            return;
        }

        // Did we click on a snap point?
        // Our snap points are always going to be on the SnapPoint physics layer

        // The normal integer number value of 8 is represented thusly in binary:
        //
        //      0000000000000000001000
        //
        // When you're talking about things like "LayerMasks" "BitMasks" "Flags"
        //
        //      00000000000000100010000 (in this case "Water" and "SnapPoint" are on)
        //      Would represent something like the 4th & 8th item in the list is enabled
        //      (The layermask is zero-indexed, so layer 4 & 8 are the 5th and 9th bit.)
        //
        // Bitwise operators and comparison
        //
        //  &: This kind of compares two integers and returns a new integer where
        //     both had their bits enabled
        //      Example:
        //              10110 
        //            & 10011
        //              ----
        //              10010
        //
        //  |: This is a bitwise or
        //
        //              10110 
        //            | 10011
        //              -----
        //              10111
        //
        //
        //  <<: The left-shift operator.
        //
        //     1 << 8
        //
        //    Goes from this: 0000000000000000001
        //    To this:        0000000000100000000
        //

        int maskForThisHitObject = 1 << theCollider.gameObject.layer;


        if( (maskForThisHitObject & snapPointLayerMask) != 0  )
        {
            // Now let's spawn a new object

            Vector3 spawnSpot = theCollider.transform.position;

            Quaternion spawnRotation = theCollider.transform.rotation; //Quaternion.FromToRotation( Vector3.forward, hitInfo.normal );

            GameObject go = (GameObject)Instantiate( thePrefabToSpawn, spawnSpot, spawnRotation );
            go.transform.SetParent(theCollider.transform);

            // Disable the renderer on the snap point while something is attached to it
            if(theCollider.GetComponent<Renderer>() != null)
            {
                theCollider.GetComponent<Renderer>().enabled = false;
            }
            if(theCollider.GetComponent<Collider>() != null)
            {
                theCollider.GetComponent<Collider>().enabled = false;
            }
        }



    }

    void RemovePart(GameObject go)
    {
        // THIS IS JUST AN EXAMPLE -- may not work right as-is

        // Make sure to re-enable the SnapPoint we are attached to

        go.transform.parent.GetComponent<Renderer>().enabled = true;
        go.transform.parent.GetComponent<Collider>().enabled = true;

        Destroy(go);
    }



}
